import os
import sys

from django.core import management


def manage(settings_file):
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', settings_file)
    management.execute_from_command_line(sys.argv)


def test(settings_file, *apps):
    optional_arguments = sys.argv[1:]
    sys.argv[1:] = ['test'] + list(apps) + optional_arguments
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', settings_file)
    management.execute_from_command_line(sys.argv)


def wsgi(settings_file, logfile=None):
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', settings_file)
    if logfile:
        import datetime

        class logger(object):
            def __init__(self, logfile):
                self.logfile = logfile

            def write(self, data):
                self.log(data)

            def writeline(self, data):
                self.log(data)

            def log(self, msg):
                line = '%s - %s\n' % (
                    datetime.datetime.now().strftime('%Y%m%d %H:%M:%S'), msg)
                fp = open(self.logfile, 'a')
                try:
                    fp.write(line)
                finally:
                    fp.close()
        sys.stdout = sys.stderr = logger(logfile)

    # Run WSGI handler for the application
    from django.core.wsgi import get_wsgi_application
    return get_wsgi_application()



import os
import sys
import unittest

import mock

from djangorecipe import binscripts


class ScriptTestCase(unittest.TestCase):

    def setUp(self):
        # We will also need to fake the settings file's module
        self.settings = mock.sentinel.Settings
        self.settings.SECRET_KEY = 'I mock your secret key'
        sys.modules['cheeseshop'] = mock.sentinel.CheeseShop
        sys.modules['cheeseshop.development'] = self.settings
        sys.modules['cheeseshop'].development = self.settings
        print("DJANGO ENV: %s" % os.environ.get('DJANGO_SETTINGS_MODULE'))

    def tearDown(self):
        # We will clear out sys.modules again to clean up
        for m in ['cheeseshop', 'cheeseshop.development']:
            del sys.modules[m]


class TestTestScript(ScriptTestCase):

    @mock.patch('django.core.management.execute_from_command_line')
    @mock.patch('os.environ.setdefault')
    def test_script(self, mock_setdefault, execute_from_command_line):
        with mock.patch.object(sys, 'argv', ['bin/test']):
            # The test script should execute the standard Django test command
            # with any apps configured in djangorecipe given as its arguments.
            binscripts.test('cheeseshop.development',  'spamm', 'eggs')
            self.assertTrue(execute_from_command_line.called)
            self.assertEqual(execute_from_command_line.call_args[0],
                             (['bin/test', 'test', 'spamm', 'eggs'],))
            self.assertEqual(mock_setdefault.call_args[0],
                             ('DJANGO_SETTINGS_MODULE',
                              'cheeseshop.development'))

    @mock.patch('django.core.management.execute_from_command_line')
    @mock.patch('os.environ.setdefault')
    def test_script_with_args(self, mock_setdefault,
                              execute_from_command_line):
        with mock.patch.object(sys, 'argv', ['bin/test', '--verbose']):
            # The test script should execute the standard Django test command
            # with any apps given as its arguments. It should also pass along
            # command line arguments so that the actual test machinery can
            # pick them up (like '--verbose' or '--tests=xyz').
            binscripts.test('cheeseshop.development',  'spamm', 'eggs')
            self.assertEqual(
                execute_from_command_line.call_args[0],
                (['bin/test', 'test', 'spamm', 'eggs', '--verbose'],))
            self.assertEqual(
                mock_setdefault.call_args[0],
                ('DJANGO_SETTINGS_MODULE', 'cheeseshop.development'))

    @mock.patch('django.core.management.execute_from_command_line')
    @mock.patch('os.environ.setdefault')
    def test_deeply_nested_settings(self, mock_setdefault,
                                    execute_from_command_line):
        # Settings files can be more than two levels deep. We need to
        # make sure the test script can properly import those. To
        # demonstrate this we need to add another level to our
        # sys.modules entries.
        settings = mock.sentinel.SettingsModule
        settings.SECRET_KEY = 'I mock your secret key'
        nce = mock.sentinel.NCE
        nce.development = settings
        sys.modules['cheeseshop'].nce = nce
        sys.modules['cheeseshop.nce'] = nce
        sys.modules['cheeseshop.nce.development'] = settings
        binscripts.test('cheeseshop.nce.development',  'tilsit', 'stilton')
        self.assertEqual(
            mock_setdefault.call_args[0],
            ('DJANGO_SETTINGS_MODULE', 'cheeseshop.nce.development'))


class TestManageScript(ScriptTestCase):

    @mock.patch('django.core.management.execute_from_command_line')
    @mock.patch('os.environ.setdefault')
    def test_script(self, mock_setdefault, mock_execute):
        # The manage script is a replacement for the default manage.py
        # script. It has all the same bells and whistles since all it
        # does is call the normal Django stuff.
        binscripts.manage('cheeseshop.development')
        self.assertEqual(mock_execute.call_args,
                         ((sys.argv,), {}))
        self.assertEqual(
            mock_setdefault.call_args,
            (('DJANGO_SETTINGS_MODULE', 'cheeseshop.development'), {}))


class TestWSGIScript(ScriptTestCase):
    # Note: don't test the logger part of wsgi(), because that overwrites
    # sys.stdout.

    def test_script(self):
        settings_dotted_path = 'cheeseshop.development'
        # ^^^ Our regular os.environ.setdefault patching doesn't help.
        # Patching get_wsgi_application already imports the DB layer, so the
        # settings are already needed there!
        with mock.patch('os.environ',
                        {'DJANGO_SETTINGS_MODULE': settings_dotted_path}):
            with mock.patch('django.core.wsgi.get_wsgi_application') \
                 as patched_method:
                binscripts.wsgi(settings_dotted_path, logfile=None)
                self.assertTrue(patched_method.called)



import os
import shutil
import sys
import tempfile
import unittest

import mock
from zc.buildout import UserError

from djangorecipe.recipe import Recipe


class BaseTestRecipe(unittest.TestCase):

    def setUp(self):
        # Create a directory for our buildout files created by the recipe
        self.buildout_dir = tempfile.mkdtemp('djangorecipe')

        self.bin_dir = os.path.join(self.buildout_dir, 'bin')
        self.develop_eggs_dir = os.path.join(self.buildout_dir,
                                             'develop-eggs')
        self.eggs_dir = os.path.join(self.buildout_dir, 'eggs')
        self.parts_dir = os.path.join(self.buildout_dir, 'parts')

        # We need to create the bin dir since the recipe should be able to
        # expect it exists
        os.mkdir(self.bin_dir)

        self.recipe_initialisation = [
            {'buildout': {
                'eggs-directory': self.eggs_dir,
                'develop-eggs-directory': self.develop_eggs_dir,
                'bin-directory': self.bin_dir,
                'parts-directory': self.parts_dir,
                'directory': self.buildout_dir,
                'python': 'buildout',
                'executable': sys.executable,
                'find-links': '',
                'allow-hosts': ''},
             },
            'django',
            {'recipe': 'djangorecipe'}]

        self.recipe = Recipe(*self.recipe_initialisation)

    def tearDown(self):
        # Remove our test dir
        shutil.rmtree(self.buildout_dir)


class TestRecipe(BaseTestRecipe):

    def test_consistent_options(self):
        # Buildout is pretty clever in detecting changing options. If
        # the recipe modifies it's options during initialisation it
        # will store this to determine wheter it needs to update or do
        # a uninstall & install. We need to make sure that we normally
        # do not trigger this. That means running the recipe with the
        # same options should give us the same results.
        self.assertEqual(Recipe(*self.recipe_initialisation).options,
                         Recipe(*self.recipe_initialisation).options)

    @mock.patch('zc.recipe.egg.egg.Scripts.working_set',
                return_value=(None, []))
    def test_update_smoketest(self, working_set):
        working_set  # noqa
        self.recipe.install()
        self.recipe.update()

    def test_create_file(self):
        # The create file helper should create a file at a certain
        # location unless it already exists. We will need a
        # non-existing file first.
        f, name = tempfile.mkstemp()
        # To show the function in action we need to delete the file
        # before testing.
        os.remove(name)
        # The method accepts a template argument which it will use
        # with the options argument for string substitution.
        self.recipe.create_file(name, 'Spam %s', 'eggs')
        # Let's check the contents of the file
        self.assertEqual(open(name).read(), 'Spam eggs')
        # If we try to write it again it will just ignore our request
        self.recipe.create_file(name, 'Spam spam spam %s', 'eggs')
        # The content of the file should therefore be the same
        self.assertEqual(open(name).read(), 'Spam eggs')
        # Now remove our temp file
        os.remove(name)

    def test_version_option_deprecation1(self):
        options = {'recipe': 'djangorecipe',
                   'version': 'trunk',
                   'wsgi': 'true'}
        self.assertRaises(UserError, Recipe, *('buildout', 'test', options))

    def test_version_option_deprecation2(self):
        options = {'recipe': 'djangorecipe',
                   'wsgilog': 'something'}
        self.assertRaises(UserError, Recipe, *('buildout', 'test', options))

    def test_version_option_deprecation3(self):
        options = {'recipe': 'djangorecipe',
                   'projectegg': 'something'}
        self.assertRaises(UserError, Recipe, *('buildout', 'test', options))

    def test_version_option_deprecation4(self):
        options = {'recipe': 'djangorecipe',
                   'deploy_script_extra': 'something'}
        self.assertRaises(UserError, Recipe, *('buildout', 'test', options))

    @mock.patch('zc.recipe.egg.egg.Scripts.working_set',
                return_value=(None, []))
    @mock.patch('djangorecipe.recipe.Recipe.create_manage_script')
    def test_extra_paths(self, manage, working_set):

        # The recipe allows extra-paths to be specified. It uses these to
        # extend the Python path within it's generated scripts.
        self.recipe.options['version'] = '1.0'
        self.recipe.options['extra-paths'] = 'somepackage\nanotherpackage'

        self.recipe.install()
        self.assertEqual(manage.call_args[0][0][-2:],
                         ['somepackage', 'anotherpackage'])

    def test_settings_option(self):
        # The settings option can be used to specify the settings file
        # for Django to use. By default it uses `development`.
        self.assertEqual(self.recipe.options['settings'], 'development')
        # When we change it an generate a manage script it will use
        # this var.
        self.recipe.options['settings'] = 'spameggs'
        self.recipe.create_manage_script([], [])
        manage = os.path.join(self.bin_dir, 'django')
        self.assertTrue("djangorecipe.binscripts.manage('project.spameggs')"
                        in open(manage).read())

    def test_dotted_settings_path_option(self):
        self.assertEqual(self.recipe.options['settings'], 'development')
        self.recipe.options['dotted-settings-path'] = 'myproj.conf.production'
        self.recipe.create_manage_script([], [])
        manage = os.path.join(self.bin_dir, 'django')
        self.assertTrue("djangorecipe.binscripts.manage('myproj.conf.production')"
                        in open(manage).read())


class TestRecipeScripts(BaseTestRecipe):

    def test_make_protocol_script_wsgi(self):
        # To ease deployment a WSGI script can be generated. The
        # script adds any paths from the `extra_paths` option to the
        # Python path.
        self.recipe.options['wsgi'] = 'true'
        self.recipe.make_wsgi_script([], [])
        # This should have created a script in the bin dir

        wsgi_script = os.path.join(self.bin_dir, 'django.wsgi')
        self.assertTrue(os.path.exists(wsgi_script))

    def test_contents_protocol_script_wsgi(self):
        self.recipe.options['wsgi'] = 'true'
        self.recipe.make_wsgi_script([], [])
        wsgi_script = os.path.join(self.bin_dir, 'django.wsgi')

        # The contents should list our paths
        contents = open(wsgi_script).read()
        # It should also have a reference to our settings module
        self.assertTrue('project.development' in contents)
        # and a line which set's up the WSGI app
        self.assertTrue("application = "
                        "djangorecipe.binscripts.wsgi('project.development', "
                        "logfile='')"
                        in contents)
        self.assertTrue("class logger(object)" not in contents)

    def test_contents_protocol_script_wsgi_with_initialization(self):
        self.recipe.options['wsgi'] = 'true'
        self.recipe.options['initialization'] = 'import os\nassert True'
        self.recipe.make_wsgi_script([], [])
        wsgi_script = os.path.join(self.bin_dir, 'django.wsgi')
        self.assertTrue('import os\nassert True\n\nimport djangorecipe'
                        in open(wsgi_script).read())

    def test_contents_log_protocol_script_wsgi(self):
        self.recipe.options['wsgi'] = 'true'
        self.recipe.options['logfile'] = '/foo'
        self.recipe.make_wsgi_script([], [])

        wsgi_script = os.path.join(self.bin_dir, 'django.wsgi')
        contents = open(wsgi_script).read()

        self.assertTrue("logfile='/foo'" in contents)

    def test_make_protocol_named_script_wsgi(self):
        # A wsgi-script name option is specified
        self.recipe.options['wsgi'] = 'true'
        self.recipe.options['wsgi-script'] = 'foo-wsgi.py'
        self.recipe.make_wsgi_script([], [])
        wsgi_script = os.path.join(self.bin_dir, 'foo-wsgi.py')
        self.assertTrue(os.path.exists(wsgi_script))

    def test_deploy_script_extra(self):
        extra_val = '#--deploy-script-extra--'
        self.recipe.options['wsgi'] = 'true'
        self.recipe.options['deploy-script-extra'] = extra_val
        self.recipe.make_wsgi_script([], [])
        wsgi_script = os.path.join(self.bin_dir, 'django.wsgi')
        contents = open(wsgi_script).read()
        self.assertTrue(extra_val in contents)

    @mock.patch('zc.buildout.easy_install.scripts',
                return_value=['some-path'])
    def test_make_protocol_scripts_return_value(self, scripts):
        # The return value of make scripts lists the generated scripts.
        self.recipe.options['wsgi'] = 'true'
        self.assertEqual(self.recipe.make_wsgi_script([], []),
                         ['some-path'])

    def test_create_manage_script(self):
        # This buildout recipe creates a alternative for the standard
        # manage.py script. It has all the same functionality as the
        # original one but it sits in the bin dir instead of within
        # the project.
        manage = os.path.join(self.bin_dir, 'django')
        self.recipe.create_manage_script([], [])
        self.assertTrue(os.path.exists(manage))

    def test_create_manage_script_with_initialization(self):
        manage = os.path.join(self.bin_dir, 'django')
        self.recipe.options['initialization'] = 'import os\nassert True'
        self.recipe.create_manage_script([], [])
        self.assertTrue('import os\nassert True\n\nimport djangorecipe'
                        in open(manage).read())

    def test_dotted_settings_path_option(self):
        self.assertEqual(self.recipe.options['settings'], 'development')
        self.recipe.options['wsgi'] = 'true'
        self.recipe.options['dotted-settings-path'] = 'myproj.conf.production'
        self.recipe.make_wsgi_script([], [])
        wsgi_script = os.path.join(self.bin_dir, 'django.wsgi')
        self.assertTrue("application = "
                        "djangorecipe.binscripts.wsgi('myproj.conf.production', "
                        "logfile='')"
                        in open(wsgi_script).read())

    def test_create_scripts_with_settings(self):
        # easy_install is available. It isn't useful, but it is a good
        # example.
        self.recipe.options['scripts-with-settings'] = 'easy_install'
        created = os.path.join(self.bin_dir, 'easy_install-with-settings')
        self.recipe.create_scripts_with_settings([], [])
        self.assertTrue(os.path.exists(created))

    def test_create_scripts_with_settings2(self):
        # easy_install is available. It isn't useful, but it is a good
        # example.
        self.recipe.options['scripts-with-settings'] = 'easy_install'
        created = os.path.join(self.bin_dir, 'easy_install-with-settings')
        self.recipe.create_scripts_with_settings([], [])
        self.assertTrue(
            "os.environ['DJANGO_SETTINGS_MODULE'] = 'project.development"
            in open(created, 'r').read())

    def test_create_scripts_with_settings3(self):
        self.recipe.options['scripts-with-settings'] = 'unavailable'
        self.assertRaises(
            UserError,  # "Script name not found"
            self.recipe.create_scripts_with_settings,
            *([], []))

    def test_create_scripts_with_settings4(self):
        # By default, nothing is generated.
        result = self.recipe.create_scripts_with_settings([], [])
        self.assertEquals([], result)


class TestTesTRunner(BaseTestRecipe):

    def test_create_test_runner(self):
        # An executable script can be generated which will make it
        # possible to execute the Django test runner. This options
        # only works if we specify one or apps to test.
        testrunner = os.path.join(self.bin_dir, 'test')

        # This first argument sets extra_paths, we will use this to
        # make sure the script can find this recipe
        recipe_dir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), '..'))

        # When we specify an app to test it should create the the
        # testrunner
        self.recipe.options['test'] = 'knight'
        self.recipe.create_test_runner([recipe_dir], [])
        self.assertTrue(os.path.exists(testrunner))

    def test_not_create_test_runner(self):
        recipe_dir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), '..'))
        self.recipe.create_test_runner([recipe_dir], [])

        testrunner = os.path.join(self.bin_dir, 'test')

        # Show it does not create a test runner by default
        self.assertFalse(os.path.exists(testrunner))

    def test_create_test_runner_with_initialization(self):
        recipe_dir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), '..'))
        testrunner = os.path.join(self.bin_dir, 'test')

        # When we specify an app to test it should create the the
        # testrunner
        self.recipe.options['test'] = 'knight'
        self.recipe.options['initialization'] = 'import os\nassert True'
        self.recipe.create_test_runner([recipe_dir], [])
        self.assertTrue('import os\nassert True\n\nimport djangorecipe'
                        in open(testrunner).read())

    def test_relative_paths_default(self):
        self.recipe.options['wsgi'] = 'true'

        self.recipe.make_wsgi_script([], [])
        self.recipe.create_manage_script([], [])

        manage = os.path.join(self.bin_dir, 'django')
        wsgi_script = os.path.join(self.bin_dir, 'django.wsgi')

        expected = 'base = os.path.dirname(os.path.abspath(os.path.realpath(__file__)))'
        self.assertFalse(expected in open(manage).read())
        self.assertFalse(expected in open(wsgi_script).read())

    def test_relative_paths_true(self):
        recipe = Recipe(
            {'buildout': {
                'eggs-directory': self.eggs_dir,
                'develop-eggs-directory': self.develop_eggs_dir,
                'python': 'python-version',
                'bin-directory': self.bin_dir,
                'parts-directory': self.parts_dir,
                'directory': self.buildout_dir,
                'find-links': '',
                'allow-hosts': '',
                'develop': '.',
                'relative-paths': 'true'},
             'python-version': {'executable': sys.executable}
         },
            'django',
            {'recipe': 'djangorecipe',
             'wsgi': 'true'})
        recipe.make_wsgi_script([], [])
        recipe.create_manage_script([], [])

        manage = os.path.join(self.bin_dir, 'django')
        wsgi_script = os.path.join(self.bin_dir, 'django.wsgi')

        expected = 'base = os.path.dirname(os.path.abspath(os.path.realpath(__file__)))'
        self.assertTrue(expected in open(manage).read())
        self.assertTrue(expected in open(wsgi_script).read())



import os
import logging
import sys

from zc.buildout import UserError
import pkg_resources
import zc.recipe.egg

from djangorecipe.boilerplate import WSGI_TEMPLATE


class Recipe(object):
    def __init__(self, buildout, name, options):
        self.log = logging.getLogger(name)

        # Deprecations
        if 'version' in options:
            raise UserError('The version option is deprecated. '
                            'Read about the change on '
                            'http://pypi.python.org/pypi/djangorecipe/0.99')
        if 'wsgilog' in options:
            raise UserError('The wsgilog option is deprecated. '
                            'Read about the change on '
                            'http://pypi.python.org/pypi/djangorecipe/2.0')
        if 'projectegg' in options:
            raise UserError("The projectegg option is deprecated. "
                            "See the changelog for 2.0 at "
                            "http://pypi.python.org/pypi/djangorecipe/2.0")
        if 'deploy_script_extra' in options:
            # Renamed between 1.9 and 1.10
            raise UserError(
                "'deploy_script_extra' option found (with underscores). "
                "This has been renamed to 'deploy-script-extra'.")
        if 'script-entrypoints' in options:
            # Renamed between 2.0 and 2.1
            raise UserError("The 'script-entrypoints' option is deprecated by "
                            "'scripts-with-settings'. See the changelog "
                            "for 2.1 at "
                            "http://pypi.python.org/pypi/djangorecipe/2.1")

        # Generic initialization.
        self.egg = zc.recipe.egg.Egg(buildout, options['recipe'], options)
        self.buildout, self.name, self.options = buildout, name, options
        options['location'] = os.path.join(
            buildout['buildout']['parts-directory'], name)
        options['bin-directory'] = buildout['buildout']['bin-directory']

        # Option defaults.
        options.setdefault('project', 'project')
        options.setdefault('settings', 'development')
        options.setdefault('extra-paths', '')
        options.setdefault('initialization', '')
        options.setdefault('deploy-script-extra', '')
        options.setdefault('scripts-with-settings', '')

        # mod_wsgi support script
        options.setdefault('wsgi', 'false')
        options.setdefault('logfile', '')

        # respect relative-paths (from zc.recipe.egg)
        relative_paths = options.get(
            'relative-paths', buildout['buildout'].get('relative-paths',
                                                       'false'))
        if relative_paths == 'true':
            options['buildout-directory'] = buildout['buildout']['directory']
            self._relative_paths = options['buildout-directory']
        else:
            self._relative_paths = ''
            assert relative_paths == 'false'

    def install(self):
        if self.options['project'] not in os.listdir(
                self.buildout['buildout']['directory']):
            # Only warn for this upon install, not on update.
            self.log.warn(
                "There's no directory named after our project. "
                "Probably you want to run 'bin/django startproject %s'",
                self.options['project'])

        extra_paths = self.get_extra_paths()
        ws = self.egg.working_set(['djangorecipe'])[1]
        # ^^^ working_set returns (requirements, ws)

        script_paths = []
        script_paths.extend(self.create_manage_script(extra_paths, ws))
        script_paths.extend(self.create_test_runner(extra_paths, ws))
        script_paths.extend(self.make_wsgi_script(extra_paths, ws))
        script_paths += self.create_scripts_with_settings(
            extra_paths, ws)
        return script_paths

    def create_manage_script(self, extra_paths, ws):
        settings = self.get_settings()
        return zc.buildout.easy_install.scripts(
            [(self.options.get('control-script', self.name),
              'djangorecipe.binscripts', 'manage')],
            ws, sys.executable, self.options['bin-directory'],
            extra_paths=extra_paths,
            relative_paths=self._relative_paths,
            arguments="'%s'" % settings,
            initialization=self.options['initialization'])

    def create_test_runner(self, extra_paths, working_set):
        settings = self.get_settings()
        apps = self.options.get('test', '').split()
        # Only create the testrunner if the user requests it
        if apps:
            return zc.buildout.easy_install.scripts(
                [(self.options.get('testrunner', 'test'),
                  'djangorecipe.binscripts', 'test')],
                working_set, sys.executable,
                self.options['bin-directory'],
                extra_paths=extra_paths,
                relative_paths=self._relative_paths,
                arguments="'%s', %s" % (
                    settings, ', '.join(["'%s'" % app for app in apps])),
                initialization=self.options['initialization'])
        else:
            return []

    def make_wsgi_script(self, extra_paths, ws):
        scripts = []
        _script_template = zc.buildout.easy_install.script_template
        settings = self.get_settings()
        zc.buildout.easy_install.script_template = (
            zc.buildout.easy_install.script_header +
            WSGI_TEMPLATE +
            self.options['deploy-script-extra']
        )
        if self.options.get('wsgi', '').lower() == 'true':
            scripts.extend(
                zc.buildout.easy_install.scripts(
                    [(self.options.get('wsgi-script') or
                      '%s.%s' % (self.options.get('control-script',
                                                  self.name),
                                 'wsgi'),
                      'djangorecipe.binscripts', 'wsgi')],
                    ws,
                    sys.executable,
                    self.options['bin-directory'],
                    extra_paths=extra_paths,
                    relative_paths=self._relative_paths,
                    arguments="'%s', logfile='%s'" % (
                        settings, self.options.get('logfile')),
                    initialization=self.options['initialization'],
                ))
        zc.buildout.easy_install.script_template = _script_template
        return scripts

    def create_scripts_with_settings(self, extra_paths, ws):
        settings = self.get_settings()
        # What we're installing here are existing setuptools entry points. We
        # look up the script names in the list of available entry points and
        # install a duplicate. We postfix the duplicate with '-with-settings',
        # so that 'gunicorn' for instance becomes 'gunicorn-with-settings'.
        postfix = '-with-settings'
        initialization = self.options['initialization']
        initialization += (
            "\n" +
            "import os\n" +
            "os.environ['DJANGO_SETTINGS_MODULE'] = '%s'" % settings)
        created_scripts = []
        script_names = [entrypoint.strip() for entrypoint in
                        self.options.get('scripts-with-settings').splitlines()
                        if entrypoint.strip()]
        known_entrypoints = list(pkg_resources.iter_entry_points(
            'console_scripts'))
        to_create = [entrypoint for entrypoint in known_entrypoints
                     if entrypoint.name in script_names]
        for entrypoint in to_create:
            script_name = entrypoint.name + postfix
            dotted_path = entrypoint.module_name
            function_name = entrypoint.attrs[0]
            self.log.debug("Creating entrypoint %s:%s as %s",
                           dotted_path, function_name, script_name)
            zc.buildout.easy_install.scripts(
                [(script_name, dotted_path, function_name)],
                ws, sys.executable, self.options['bin-directory'],
                extra_paths=extra_paths,
                relative_paths=self._relative_paths,
                initialization=initialization)
            created_scripts.append(script_name)

        # Feedback logging.
        known_names = [entrypoint.name for entrypoint in known_entrypoints]
        unkown_script_names = [name for name in script_names
                               if name not in known_names]
        if unkown_script_names:
            raise UserError("Some script names couldn't be found: %s" % (
                ', '.join(unkown_script_names)))

        return created_scripts

    def get_extra_paths(self):
        extra_paths = [self.buildout['buildout']['directory']]
        pythonpath = [p.replace('/', os.path.sep) for p in
                      self.options['extra-paths'].splitlines() if p.strip()]
        extra_paths.extend(pythonpath)
        return extra_paths

    def update(self):
        extra_paths = self.get_extra_paths()
        ws = self.egg.working_set(['djangorecipe'])[1]
        # ^^^ working_set returns (requirements, ws)

        self.create_manage_script(extra_paths, ws)
        self.create_test_runner(extra_paths, ws)
        self.make_wsgi_script(extra_paths, ws)

    def create_file(self, filename, template, options):
        if os.path.exists(filename):
            return
        f = open(filename, 'w')
        f.write(template % options)
        f.close()

    def get_settings(self):
        settings = '%s.%s' % (self.options['project'], self.options['settings'])
        settings = self.options.get('dotted-settings-path', settings)
        return settings
